@@ -9,17 +9,18 @@
 */
 
 /* .Call calls */
-extern SEXP dWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP dtdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP dadWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP dvdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP dwdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dtdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dadWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dvdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dwdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dsvdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP pWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP dapWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP dvpWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP dwpWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 
-extern SEXP dxdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP dxdWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP dxpWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 
 extern SEXP dDiffusion7(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -32,17 +33,18 @@ extern SEXP randWiener(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEX
 
 
 static const R_CallMethodDef CallEntries[] = {
-    {"dWiener", (DL_FUNC) &dWiener, 10},
-    {"dtdWiener", (DL_FUNC) &dtdWiener, 10},
-    {"dadWiener", (DL_FUNC) &dadWiener, 10},
-    {"dvdWiener", (DL_FUNC) &dvdWiener, 10},
-    {"dwdWiener", (DL_FUNC) &dwdWiener, 10},
+    {"dWiener", (DL_FUNC) &dWiener, 11},
+    {"dtdWiener", (DL_FUNC) &dtdWiener, 11},
+    {"dadWiener", (DL_FUNC) &dadWiener, 11},
+    {"dvdWiener", (DL_FUNC) &dvdWiener, 11},
+    {"dwdWiener", (DL_FUNC) &dwdWiener, 11},
+    {"dsvdWiener", (DL_FUNC) &dsvdWiener, 11},
     {"pWiener", (DL_FUNC) &pWiener, 10},
     {"dapWiener", (DL_FUNC) &dapWiener, 10},
     {"dvpWiener", (DL_FUNC) &dvpWiener, 10},
     {"dwpWiener", (DL_FUNC) &dwpWiener, 10},
 
-    {"dxdWiener", (DL_FUNC) &dxdWiener, 10},
+    {"dxdWiener", (DL_FUNC) &dxdWiener, 11},
     {"dxpWiener", (DL_FUNC) &dxpWiener, 10},
 
     {"dDiffusion7", (DL_FUNC) &dDiffusion7, 16},
